# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import fields, models


class ScopAccountInvoiceIDFReport(models.Model):
    _inherit = "account.invoice.all"

    source = fields.Selection(
        selection_add=[("idf", "IDF")], ondelete={"idf": "cascade"}
    )

    # ------------------------------------------------------
    # Query
    # ------------------------------------------------------
    def _subquery(self):
        select_str = super(ScopAccountInvoiceIDFReport, self)._subquery()
        select_str += """
                UNION ALL
                SELECT
                    'idf' as source,
                    null as invoice_id,
                    writing_date as invoice_date,
                    writing_date as invoice_date_due,
                    acc_doc as name,
                    company_id,
                    partner_id,
                    null as invoice_user_id,
                    debit as amount_untaxed_signed,
                    debit as amount_total_signed,
                    amount_residual as amount_residual_signed,
                    'out_invoice' as move_type,
                    'posted' as state,
                    CASE
                        WHEN amount_residual = 0 THEN 'paid'
                        WHEN amount_residual = debit THEN 'not_paid'
                        ELSE 'partial'
                    END AS payment_state
                FROM
                    scop_invoice_idf
                WHERE
                    type = 'inv'
        """
        return select_str

    # ------------------------------------------------------
    # Computed fields
    # ------------------------------------------------------
    def _compute_lines(self):
        for inv in self:
            if inv.source == "idf":
                invoice_idf_id = (
                    self.env["scop.invoice.idf"]
                    .sudo()
                    .search(
                        [
                            ("partner_id", "=", inv.partner_id.id),
                            ("writing_date", "=", inv.invoice_date),
                            ("acc_doc", "=", inv.name),
                            ("type", "=", "inv"),
                        ]
                    )
                )
                if len(invoice_idf_id) == 1:
                    inv.lines = (
                        "<table class='table table-hover table-striped table-sm'>"
                        "<thead class='thead-light'><tr><th>Description</th>"
                        "<th>Quantité</th><th>P.U</th>"
                        "<th>Total HT</th><th>Total TTC</th></tr></thead><tbody>"
                        + "".join(
                            invoice_idf_id.mapped(
                                lambda l: (
                                    "<tr><td>"
                                    + l.name.replace("\n", "<br/>")
                                    + "</td>"
                                    + "<td></td>"
                                    + "<td>"
                                    + str(l.debit)
                                    + "</td>"
                                    + "<td>"
                                    + str(l.debit)
                                    + "</td>"
                                    + "<td>"
                                    + str(l.debit)
                                    + "</td></tr>"
                                )
                            )
                        )
                        + "</tbody></table>"
                    )
            if not inv.lines:
                super(ScopAccountInvoiceIDFReport, inv)._compute_lines()
